@@ -2,7 +2,7 @@
 ##
 ##  pyvnc2swf - output.py
 ##
-##  $Id: output.py,v 1.2 2007/04/25 16:55:25 euske Exp $
+##  $Id: output.py,v 1.8 2008/11/16 02:39:40 euske Exp $
 ##
 ##  Copyright (C) 2005 by Yusuke Shinyama (yusuke at cs . nyu . edu)
 ##  All Rights Reserved.
@@ -112,7 +112,7 @@ class SWFScreen:
       # image defined by SCREENVIDEOPACKET (BGR)
       data = zlib.decompress(data)
       assert len(data) == (w*h*3)
-      img = create_image_from_string_bgr_flipped(w, h, data)
+      img = create_image_from_string_rgb_flipped(w, h, bgr2rgb(data))
     else:
       assert 0, 'illegal image format: %d' % format
     # sometime the pasted image doesn't fit in the screen, but just let it out.
@@ -231,6 +231,8 @@ class SWFVideoScreen(SWFBlockScreen):
 
   # must return a string!
   def get_block_change(self, x, y):
+    # okay, this function is going to be called millions of times.
+    # so it must perform ultra-fast.
     '''get change of block (x,y)'''
     if not self.block_changed[y][x]:
       return ''
@@ -242,7 +244,7 @@ class SWFVideoScreen(SWFBlockScreen):
     if y0 < 0:
       h += y0
       y0 = 0
-    # for some reason y-axis is filpped in VideoPacket. (BGR)
+    # for some reason y-axis is filpped in VideoPacket.
     # so we flip it in advance so that it can go back correctly...
     data = convert_image_to_string_rgb_flipped(self.get_image(x0, y0, w, h))
     hval = hash(data)
@@ -250,8 +252,7 @@ class SWFVideoScreen(SWFBlockScreen):
       return ''
     self.block_changed[y][x] = False
     self.block_image[y][x] = hval
-    data = ''.join([ data[i+2]+data[i+1]+data[i] for i in xrange(0, len(data), 3) ])
-    return data
+    return bgr2rgb(data)
   
   def paint_image(self, x0, y0, w, h, data):
     if not SWFScreen.paint_image(self, x0, y0, w, h, data): return False
@@ -646,9 +647,9 @@ class SWFVideoStream(SWFOutputStream):
         self.writer.writebits(4, self.screen.block_h/16-1)
         self.writer.writebits(12, self.screen.out_height)
         self.writer.finishbits()
-        for blocks in r:
-          if blocks:
-            data = zlib.compress(''.join(blocks))
+        for data in r:
+          if data:
+            data = zlib.compress(data)
             self.writer.writeub16(len(data))
             self.writer.write(data)
           else:
@@ -828,6 +829,7 @@ class FLVVideoStream(MovieOutputStream):
   def __init__(self, info, debug=0):
     assert info.filename, 'Filename not specified!'
     MovieOutputStream.__init__(self, info, debug)
+    self.info.set_swf_version(None)
     return
   
   def open(self):
@@ -840,7 +842,6 @@ class FLVVideoStream(MovieOutputStream):
     self.screen = SWFVideoScreen(x, y, w, h, self.info.blocksize, self.info.blocksize,
                                  scaling=self.info.scaling)
     self.set_keyframe()
-    self.painted = False
     return
 
   def paint_frame(self, (images, othertags, cursor_info)):
@@ -849,55 +850,47 @@ class FLVVideoStream(MovieOutputStream):
     for ((x0,y0), (w,h,data)) in images:
       if self.debug:
         print >>stderr, 'paint:', (x0,y0), (w,h)
-      if self.screen.paint_image(x0, y0, w, h, data):
-        self.painted = True
+      self.screen.paint_image(x0, y0, w, h, data)
     if cursor_info:
       (cursor_image, cursor_pos) = cursor_info
       if cursor_image:
         (w, h, dx, dy, data) = cursor_image
         self.cursor_offset = (dx, dy)
         self.cursor_image = create_image_from_string_argb(w, h, data)
-        self.painted = True
       if cursor_pos:
         self.cursor_pos = cursor_pos
-        self.painted = True
     return
 
   def next_frame(self):
-    if self.is_keyframe or self.painted:
-      r = []
-      changed = self.is_keyframe
-      self.screen.prepare_image(self.cursor_image, self.cursor_offset, self.cursor_pos)
-      for y in xrange(self.screen.vblocks):
-        for x in xrange(self.screen.hblocks):
-          data = self.screen.get_block_change(x, y)
-          r.append(data)
-          if data:
-            changed = True
-      if changed:
-        # write FLV tag
-        self.writer.start_tag()
-        # SCREENVIDEOPACKET
-        if self.is_keyframe:
-          self.writer.writebits(4, 1)
-          self.is_keyframe = False
-        else:
-          self.writer.writebits(4, 2)
-        self.writer.writebits(4, 3) # screenvideo codec
-        self.writer.writebits(4, self.screen.block_w/16-1)
-        self.writer.writebits(12, self.screen.out_width)
-        self.writer.writebits(4, self.screen.block_h/16-1)
-        self.writer.writebits(12, self.screen.out_height)
-        self.writer.finishbits()
-        for blocks in r:
-          if blocks:
-            data = zlib.compress(''.join(blocks))
-            self.writer.writeub16(len(data))
-            self.writer.write(data)
-          else:
-            self.writer.writeub16(0)
-        t = (self.output_frames*1000+1) / self.info.framerate
-        self.writer.end_tag(9, t)
+    r = []
+    self.screen.prepare_image(self.cursor_image, self.cursor_offset, self.cursor_pos)
+    for y in xrange(self.screen.vblocks):
+      for x in xrange(self.screen.hblocks):
+        r.append(self.screen.get_block_change(x, y))
+    # write FLV tag
+    self.writer.start_tag()
+    # SCREENVIDEOPACKET
+    if self.is_keyframe:
+      self.writer.writebits(4, 1)
+      self.is_keyframe = False
+    else:
+      self.writer.writebits(4, 2)
+    self.writer.writebits(4, 3) # screenvideo codec
+    self.writer.writebits(4, self.screen.block_w/16-1)
+    self.writer.writebits(12, self.screen.out_width)
+    self.writer.writebits(4, self.screen.block_h/16-1)
+    self.writer.writebits(12, self.screen.out_height)
+    self.writer.finishbits()
+    for data in r:
+      if data:
+        data = zlib.compress(data)
+        self.writer.writeub16(len(data))
+        self.writer.write(data)
+      else:
+        self.writer.writeub16(0)
+    # the first tag: always t == 0
+    t = (self.output_frames*1000) / self.info.framerate
+    self.writer.end_tag(9, t)
     MovieOutputStream.next_frame(self)
     return
 
@@ -913,6 +906,18 @@ class FLVVideoStream(MovieOutputStream):
     return
 
 
+##  StreamFactory
+##
+def StreamFactory(type):
+  return {
+    'flv': FLVVideoStream,
+    'swf5': SWFShapeStream,
+    'swf7': SWFVideoStream,
+    'mpeg': MPEGVideoStream,
+    'image': ImageSequenceStream,
+    }[type]
+
+
 ##  MovieBuilder
 ##
 class MovieBuilder:
