@@ -2,7 +2,7 @@
 ##
 ##  pyvnc2swf - rfb.py
 ##
-##  $Id: rfb.py,v 1.2 2007/04/25 16:55:25 euske Exp $
+##  $Id: rfb.py,v 1.5 2008/11/16 02:39:40 euske Exp $
 ##
 ##  Copyright (C) 2005 by Yusuke Shinyama (yusuke at cs . nyu . edu)
 ##  All Rights Reserved.
@@ -540,12 +540,12 @@ class RFBNetworkClient(RFBProxy):
 ##
 class RFBNetworkClientForRecording(RFBNetworkClient):
   
-  def __init__(self, host, port, fname, pwdfile=None,
+  def __init__(self, host, port, fp, pwdfile=None,
                preferred_encoding=(5,0), debug=0):
     RFBNetworkClient.__init__(self, host, port, fb=None, pwdfile=pwdfile,
                               preferred_encoding=preferred_encoding, debug=debug)
-    print >>stderr, 'Creating vncrec: %r: vncLog0.0' % fname
-    self.fp = file(fname, 'wb')
+    print >>stderr, 'Creating vncrec: %r: vncLog0.0' % fp
+    self.fp = fp
     self.write('vncLog0.0')
     # disguise data (security=none)
     self.write('RFB 003.003\x0a')
@@ -561,7 +561,7 @@ class RFBNetworkClientForRecording(RFBNetworkClient):
     if self.updated:
       self.updated = False
       t = time.time()
-      self.write(pack('>LL', int(t), (t-int(t))*1000000))
+      self.write(pack('>LL', int(t), int((t-int(t))*1000000)))
       RFBNetworkClient.request_update(self)
     return
   
@@ -574,22 +574,16 @@ class RFBNetworkClientForRecording(RFBNetworkClient):
     self.write(data)
     return data
 
-  def close(self):
-    RFBNetworkClient.close(self)
-    self.fp.close()
-    return
-
 
 ##  RFBFileParser
 ##
 class RFBFileParser(RFBProxy):
   
-  def __init__(self, fname, fb=None, debug=0):
+  def __init__(self, fp, fb=None, debug=0):
     RFBProxy.__init__(self, fb=fb, debug=debug)
     if self.fb:
       self.fb.change_format = False
-    self.fp = file(fname, 'rb')
-    self.fname = fname
+    self.fp = fp
     return
 
   def preferred_format(self, bitsperpixel, depth, bigendian, truecolour,
@@ -638,7 +632,7 @@ class RFBFileParser(RFBProxy):
   def init(self):
     self.curtime = 0
     version = self.fp.read(9)
-    print >>stderr, 'Reading vncrec file: %s, version=%r...' % (self.fname, version)
+    print >>stderr, 'Reading vncrec file: %s, version=%r...' % (self.fp, version)
     if version != 'vncLog0.0':
       raise RFBProtocolError('Unsupported vncrec version: %r' % version)
     return RFBProxy.init(self)
@@ -653,16 +647,18 @@ class RFBFileParser(RFBProxy):
     return
 
   def auth(self):
-    if self.protocol_version == 3:
+    # XXX for some reason, we force it act as ver 3.3 client.
+    if self.protocol_version == 3 or True:
       # protocol 3.3
       # recv: server security
       (server_security,) = unpack('>L', self.recv(4))
       if self.debug:
-        print >>stderr, 'server_security=%r' % server_security
+        print >>stderr, 'server_security: %r' % server_security
       if server_security == 2:
         # skip challenge+result (dummy)
         self.recv(20)
     else:
+      # protocol 3.7 or 3.8
       RFBProxy.auth(self)
     return self
 
@@ -767,9 +763,10 @@ class RFBMovieConverter(RFBConverter):
           done = True
     return
 
-  def open(self, fname, debug=0):
+  def open(self, fname):
     self.processing = False
-    self.rfbparser = RFBFileParser(fname, self, debug)
+    fp = file(fname, 'rb')
+    self.rfbparser = RFBFileParser(fp, self, self.debug)
     self.rfbparser.init().auth().start()
     self.beginpos = self.rfbparser.tell()
     self.rfbparser.loop()
