@@ -2,7 +2,7 @@
 ##
 ##  pyvnc2swf - swf.py
 ##
-##  $Id: swf.py,v 1.2 2007/04/25 16:55:25 euske Exp $
+##  $Id: swf.py,v 1.6 2008/11/16 02:39:40 euske Exp $
 ##
 ##  Copyright (C) 2005 by Yusuke Shinyama (yusuke at cs . nyu . edu)
 ##  All Rights Reserved.
@@ -163,7 +163,11 @@ class SWFParser(DataParser):
         # branch to scan_tag<N>
         if hasattr(self, name):
           getattr(self, name)(tag, length)
-        self.fp.seek(pos0+length)
+        if self.debug:
+          data = self.fp.read(length)
+          print >>stderr, 'tag=%d, data=%r' % (tag, data)
+        else:
+          self.fp.seek(pos0+length)
         if tag == 1:
           # next frame
           self.framepos.append(pos)
@@ -174,8 +178,7 @@ class SWFParser(DataParser):
 
   def parse_header(self):
     (F,W,S,V) = self.read(4)
-    assert W == 'W'
-    assert S == 'S'
+    assert W+S == 'WS'
     self.swf_version = ord(V)
     if 6 <= self.swf_version:
       self.encoding = 'utf-8'
@@ -393,7 +396,7 @@ class FLVParser(DataParser):
       while 1:
         tag = self.readui8()
         length = self.readub24()
-        timestamp = self.readub24()
+        timestamp = self.readub24()     # timestamp in msec.
         reserved = self.readub32()
         offset = self.fp.tell()
         self.tags.append((tag, length, timestamp, offset))
@@ -404,9 +407,7 @@ class FLVParser(DataParser):
 
   def parse_header(self):
     (F,L,V,ver) = self.read(4)
-    assert F == 'F'
-    assert L == 'L'
-    assert V == 'V'
+    assert F+L+V == 'FLV'
     self.flv_version = ord(ver)
     flags = self.readui8()
     offset = self.readub32()
@@ -419,7 +420,19 @@ class FLVParser(DataParser):
     self.fp.seek(offset)
     data = self.read(length)
     return (tag, timestamp, data)
-    
+
+  def seek_tag(self, t):
+    i0 = 0
+    i1 = len(self.tags)
+    while i0 < i1:
+      i = (i0+i1)/2
+      (tag, length, timestamp, offset) = self.tags[i]
+      if timestamp < t:
+        i0 = i
+      else:
+        i1 = i
+    return (tag, length, timestamp, offset)
+  
 
 ##  SWFWriter
 ##
@@ -756,6 +769,8 @@ class SWFWriter(DataWriter):
   """
 
   def __init__(self, outfile, swf_version, rect, framerate, compression):
+    if outfile == '-':
+      raise VelueError('cannot write a SWF file to stdout.')
     self.outfp = file(outfile, 'wb')
     self.swf_version = swf_version
     self.rect = rect
@@ -819,7 +834,10 @@ class SWFWriter(DataWriter):
 class FLVWriter(DataWriter):
   
   def __init__(self, outfile, flv_version, rect, framerate):
-    self.outfp = file(outfile, 'wb')
+    if outfile == '-':
+      self.outfp = sys.stdout
+    else:
+      self.outfp = file(outfile, 'wb')
     self.rect = rect
     self.flv_version = flv_version
     self.framerate = framerate
@@ -828,7 +846,6 @@ class FLVWriter(DataWriter):
     self.buff = 0
     self.fp = self.outfp
     self.fp.write('FLV%c' % self.flv_version)
-    self.lenpos = self.fp.tell()
     self.writebits(5,0)
     self.writebits(1,0) # has audio
     self.writebits(1,0)
@@ -842,7 +859,7 @@ class FLVWriter(DataWriter):
     data = self.pop()
     self.writeui8(tag)
     self.writeub24(len(data))
-    self.writeub24(timestamp)
+    self.writeub24(int(timestamp))
     self.writeui32(0)   # reserved
     self.write(data)
     self.writeub32(len(data)+11)  #size of this tag
@@ -855,4 +872,5 @@ class FLVWriter(DataWriter):
 
 # test
 if __name__ == "__main__":
-  FLVParser(True).open(sys.argv[1])
+  parser = FLVParser(True)
+  parser.open(sys.argv[1])
