@@ -2,7 +2,7 @@
 ##
 ##  pyvnc2swf - vnc2swf.py
 ##
-##  $Id: vnc2swf.py,v 1.1 2007/03/31 04:02:16 euske Exp $
+##  $Id: vnc2swf.py,v 1.7 2008/11/16 02:39:40 euske Exp $
 ##
 ##  Copyright (C) 2005 by Yusuke Shinyama (yusuke at cs . nyu . edu)
 ##  All Rights Reserved.
@@ -30,8 +30,8 @@ from struct import pack, unpack
 import threading
 
 from movie import SWFInfo
-from output import SWFShapeStream, SWFVideoStream
-from rfb import RFBError, RFBNetworkClient, RFBNetworkClientForRecording, RFBStreamConverter
+from output import StreamFactory
+from rfb import RFBError, RFBNetworkClient, RFBFileParser, RFBNetworkClientForRecording, RFBStreamConverter
 stderr = sys.stderr
 
 
@@ -132,13 +132,15 @@ class RFBNetworkClientForRecordingWithTk(RFBNetworkClientWithTkMixin, RFBNetwork
 class VNC2SWFWithTk:
 
   FILE_TYPES = [
-    ('SWF(Shape)', 'shape', 'Macromedia Flash Files', '.swf'),    # 0
-    ('SWF(Video)', 'video', 'Macromedia Flash Files', '.swf'),    # 1
-    ('VNCRec', 'vnc', 'VNCRec Files', '.vnc'),          # 2
+    ('Flash(v5)', 'swf5',  'Macromedia Flash Files', '.swf'),    # 0
+    ('Flash(v7)', 'swf7',  'Macromedia Flash Files', '.swf'),    # 1
+    ('FLV',       'flv',   'Macromedia Flash Video Files', '.flv'), # 3
+    ('MPEG',      'mpeg',  'MPEG Files', '.mpeg'),               # 2
+    ('VNCRec',    'vnc',   'VNCRec Files', '.vnc'),              # 4
     ]
 
   def __init__(self, tempdir, info,
-               outtype='shape', host='localhost', port=5900,
+               outtype='swf5', host='localhost', port=5900,
                preferred_encoding=(0,), subprocess=None, pwdfile=None,
                debug=0):
     self.tempdir = tempdir
@@ -297,16 +299,15 @@ class VNC2SWFWithTk:
     else:
       moviefile = os.path.join(self.tempdir, 'pyvnc2swf-%d%s' % (os.getpid(), ext))
     self.info.filename = moviefile
+    self.fp = None
     if self.outtype == 'vnc':
+      self.fp = file(self.info.filename, 'wb')
       self.client = RFBNetworkClientForRecordingWithTk(
-        self.host, self.port, self.info.filename, pwdfile=self.pwdfile,
+        self.host, self.port, self.fp, pwdfile=self.pwdfile,
         preferred_encoding=self.preferred_encoding)
       self.stream = None
     else:
-      if self.outtype == 'shape':
-        self.stream = SWFShapeStream(self.info)
-      else:
-        self.stream = SWFVideoStream(self.info)
+      self.stream = StreamFactory(self.outtype)(self.info)
       self.client = RFBNetworkClientWithTk(
         self.host, self.port, RFBStreamConverter(self.info, self.stream),
         pwdfile=self.pwdfile,
@@ -327,6 +328,9 @@ class VNC2SWFWithTk:
       # Finish the movie.
       self.stream.close()
       self.stream = None
+      if self.fp:
+        self.fp.close()
+        self.fp = None
     shutil.move(self.info.filename, filename)
     self.info.write_html(filename=filename)
     self.set_status()
@@ -349,26 +353,29 @@ class VNC2SWFWithTk:
     self.client.tk_init(self.root)
     try:
       self.client.init().auth().start()
-      if self.debug:
-        print 'start recording'
-      self.recording = True
-      self.set_status()
-      if self.subprocess:
-        self.subprocess.start()
-      try:
-        self.client.loop()
-      finally:
-        if self.debug:
-          print 'stop recording'
-        if self.subprocess:
-          self.subprocess.stop()
-        self.client.close()
-        self.recording = False
-        self.set_status()
     except socket.error, e:
       return self.error('Socket error', e)
     except RFBError, e:
       return self.error('RFB protocol error', e)
+    if self.debug:
+      print >>stderr, 'start recording'
+    self.recording = True
+    self.set_status()
+    if self.subprocess:
+      self.subprocess.start()
+    try:
+      self.client.loop()
+    except socket.error, e:
+      return self.error('Socket error', e)
+    except RFBError, e:
+      return self.error('RFB protocol error', e)
+    if self.debug:
+      print >>stderr, 'stop recording'
+    if self.subprocess:
+      self.subprocess.stop()
+    self.client.close()
+    self.recording = False
+    self.set_status()
     if self.exit_immediately:
       self.file_exit()
     return
@@ -387,47 +394,56 @@ class VNC2SWFWithTk:
 
 ##  vnc2swf - CLI routine
 ##
-def vnc2swf(info, outtype='shape', host='localhost', port=5900,
-            preferred_encoding=(0,), subprocess=None, pwdfile=None,
+def vnc2swf(info, outtype='swf5', host='localhost', port=5900, 
+            preferred_encoding=(0,), subprocess=None, pwdfile=None, vncfile=None,
             debug=0, merge=False):
-  if outtype in ('shape','video'):
-    if outtype == 'shape':
-      stream = SWFShapeStream(info)
+  fp = None
+  if outtype == 'vnc':
+    if info.filename == '-':
+      fp = sys.stdout
     else:
-      stream = SWFVideoStream(info)
-    converter = RFBStreamConverter(info, stream)
-    client = RFBNetworkClient(host, port, converter, pwdfile=pwdfile,
-                              preferred_encoding=preferred_encoding)
+      fp = file(info.filename, 'wb')
+    client = RFBNetworkClientForRecording(host, port, fp, pwdfile=pwdfile,
+                                          preferred_encoding=preferred_encoding, debug=debug)
   else:
-    client = RFBNetworkClientForRecording(host, port, info.filename, pwdfile=pwdfile,
-                                          preferred_encoding=preferred_encoding)
+    stream = StreamFactory(outtype)(info, debug=debug)
+    converter = RFBStreamConverter(info, stream, debug=debug)
+    if vncfile:
+      client = RFBFileParser(vncfile, converter, debug=debug)
+    else:
+      client = RFBNetworkClient(host, port, converter, pwdfile=pwdfile,
+                                preferred_encoding=preferred_encoding, debug=debug)
   try:
     client.init().auth().start()
-    if debug:
-      print 'start recording'
-    if subprocess:
-      subprocess.start()
-    try:
-      try:
-        client.loop()
-      except KeyboardInterrupt:
-        pass
-    finally:
-      if debug:
-        print 'stop recording'
-      if subprocess:
-        subprocess.stop()
-      client.close()
-      stream.close()
-      info.write_html()
   except socket.error, e:
     print >>stderr, 'Socket error:', e
   except RFBError, e:
     print >>stderr, 'RFB error:', e
+  if debug:
+    print >>stderr, 'start recording'
+  if subprocess:
+    subprocess.start()
+  try:
+    client.loop()
+  except KeyboardInterrupt:
+    pass
+  except socket.error, e:
+    print >>stderr, 'Socket error:', e
+  except RFBError, e:
+    print >>stderr, 'RFB error:', e
+  if debug:
+    print >>stderr, 'stop recording'
+  if subprocess:
+    subprocess.stop()
+  client.close()
+  stream.close()
+  info.write_html()
+  if fp:
+    fp.close()
   # Contributed by David Fraser
   if merge:
     tmpfile = os.tempnam(os.path.dirname(info.filename), "vncmerge-") + os.path.basename(info.filename)
-    print "renaming %s to %s for merge" % (info.filename, tmpfile)
+    print >>stderr, "renaming %s to %s for merge" % (info.filename, tmpfile)
     if os.path.exists(tmpfile):
       os.remove(tmpfile)
     os.rename( info.filename, tmpfile )
@@ -440,7 +456,7 @@ def vnc2swf(info, outtype='shape', host='localhost', port=5900,
         print >>stderr, "Error doing merge..."
     finally:
       origexists, tmpexists = os.path.exists(info.filename), os.path.exists(tmpfile)
-      print "origexists %r, tmpexists %r" % (origexists, tmpexists)
+      print >>stderr, "origexists %r, tmpexists %r" % (origexists, tmpexists)
       if origexists and tmpexists:
         try:
           os.remove(tmpfile)
@@ -458,7 +474,7 @@ class RecordingThread:
     try:
       import record_sound
     except ImportError, e:
-      print "unable to use pymedia?:", e
+      print >>stderr, "unable to use pymedia?:", e
       raise
     self.outputfile = outputfile
     self.recorder = record_sound.voiceRecorder(self.outputfile)
@@ -501,18 +517,18 @@ class Subprocess:
 
 # main
 # ./vnc2swf.py -S 'arecord -t wav -c 1 -r 22050 out.wav' -n -o out.swf 
-def main(args):
+def main(argv):
   import getopt
   def usage():
-    print ("usage: vnc2swf.py [-d] [-n] [-o filename] [-t {shape|video|vnc}]" \
-           " [-e encoding] [-N] [-C clipping] [-r framerate] [-z] [-m] [-a]" \
-           " [-S subprocess] [-P pwdfile] [host[:display] [port]]")
-    sys.exit(2)
+    print ('usage: %s [-d] [-n] [-o filename] [-t {flv|mpeg|swf5|swf7|vnc}]'
+           ' [-e encoding] [-N] [-C clipping] [-r framerate] [-s scaling] [-z] [-m] [-a] [-V]'
+           ' [-S subprocess] [-P pwdfile] [host[:display] [port]]' % argv[0])
+    return 100
   try:
-    (opts, args) = getopt.getopt(sys.argv[1:], 'dno:t:e:NC:r:S:P:zma')
+    (opts, args) = getopt.getopt(argv[1:], 'dno:t:e:NC:r:S:P:s:zmaV')
   except getopt.GetoptError:
-    usage()
-  (debug, console, outtype, subprocess, merge, pwdfile) = (0, False, None, None, False, None)
+    return usage()
+  (debug, console, outtype, subprocess, merge, pwdfile, isfile) = (0, False, None, None, False, None, False)
   (cursor, host, port, preferred_encoding) = (True, 'localhost', 5900, (0,))
   info = SWFInfo()
   for (k, v) in opts:
@@ -525,29 +541,37 @@ def main(args):
     elif k == '-a': subprocess = RecordingThread(v)
     elif k == '-m': merge = True
     elif k == '-P': pwdfile = v
+    elif k == '-V': isfile = True
     elif k == '-o':
       info.filename = v
     elif k == '-C':
       try:
         info.set_clipping(v)
       except ValueError:
-        print >>stderr, 'Invalid clipping specification:', v
-        usage()
+        print 'Invalid clipping specification:', v
+        return usage()
     elif k == "-r":
       info.framerate = int(v)
     elif k == "-z":
       info.set_scalable(True)
+    elif k == '-s':
+      info.scaling = float(v)
+      assert 0 < info.scaling and info.scaling <= 1.0, 'Invalid scaling.'
   if not outtype:
     if info.filename:
       if info.filename.endswith('.vnc'):
         outtype = 'vnc'
       elif info.filename.endswith('.swf'):
-        outtype = 'shape'
+        outtype = 'swf5'
+      elif info.filename.endswith('.mpg') or info.filename.endswith('.mpeg'):
+        outtype = 'mpeg'
+      elif info.filename.endswith('.flv'):
+        outtype = 'flv'
     else:
-      outtype = 'shape'
-  if outtype not in ('shape','video','vnc'):
+      outtype = 'swf5'
+  if outtype not in ('swf5','swf7','vnc','mpeg','flv'):
     print 'Please specify the output type or file extension.'
-    usage()
+    return usage()
   if cursor:
     preferred_encoding += (-232,-239,)
   if 1 <= len(args):
@@ -562,10 +586,15 @@ def main(args):
   if console:
     if not info.filename:
       print 'Please specify the output filename.'
-      usage()
+      return usage()
+    vncfile = None
+    if isfile:
+      vncfile = sys.stdin
+      if args:
+        vncfile = file(args[0], 'rb')
     vnc2swf(info, outtype, host, port,
             preferred_encoding=preferred_encoding,
-            subprocess=subprocess, pwdfile=pwdfile,
+            subprocess=subprocess, pwdfile=pwdfile, vncfile=vncfile,
             merge=merge, debug=debug)
   else:
     tempdir = os.path.join(tempfile.gettempdir(), 'pyvnc2swf')
@@ -577,5 +606,6 @@ def main(args):
                   preferred_encoding=preferred_encoding,
                   subprocess=subprocess, pwdfile=pwdfile,
                   debug=debug).run()
+  return
 
-if __name__ == "__main__": main(sys.argv[1:])
+if __name__ == "__main__": sys.exit(main(sys.argv))
